@@ -3,7 +3,6 @@
 
 """A base class for creating simple composable actors."""
 
-from . import power_distributing
 from ._channel_registry import ChannelRegistry
 from ._config_managing import ConfigManagingActor
 from ._data_sourcing import ComponentMetricRequest, DataSourcingActor
@@ -17,5 +16,4 @@ __all__ = [
     "ConfigManagingActor",
     "DataSourcingActor",
     "actor",
-    "power_distributing",
 ]
